@@ -1,34 +1,36 @@
-# copyright 2003-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This file is part of astroid.
-#
-# astroid is free software: you can redistribute it and/or modify it
-# under the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 2.1 of the License, or (at your
-# option) any later version.
-#
-# astroid is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
-# for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along
-# with astroid. If not, see <http://www.gnu.org/licenses/>.
+# -*- coding: utf-8 -*-
+# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2015 Google, Inc.
+# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
+# Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
+# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2017 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2018 brendanator <brendan.maginnis@gmail.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+
 """tests for the astroid builder and rebuilder module"""
 
+import builtins
+import collections
 import os
 import sys
 import unittest
 
-from astroid import builder, nodes, InferenceError, NotFoundError
-from astroid.bases import YES, BUILTINS
-from astroid.manager import AstroidManager
+from astroid import builder
+from astroid import exceptions
+from astroid import manager
+from astroid import nodes
 from astroid import test_utils
+from astroid import util
 from astroid.tests import resources
 
-MANAGER = AstroidManager()
-IS_PY3 = sys.version_info[0] == 3
+MANAGER = manager.AstroidManager()
+BUILTINS = builtins.__name__
 
 
 class FromToLineNoTest(unittest.TestCase):
@@ -42,11 +44,11 @@ class FromToLineNoTest(unittest.TestCase):
         #    function('aeozrijz\
         #    earzer', hop)
         discard = stmts[0]
-        self.assertIsInstance(discard, nodes.Discard)
+        self.assertIsInstance(discard, nodes.Expr)
         self.assertEqual(discard.fromlineno, 4)
         self.assertEqual(discard.tolineno, 5)
         callfunc = discard.value
-        self.assertIsInstance(callfunc, nodes.CallFunc)
+        self.assertIsInstance(callfunc, nodes.Call)
         self.assertEqual(callfunc.fromlineno, 4)
         self.assertEqual(callfunc.tolineno, 5)
         name = callfunc.func
@@ -71,11 +73,11 @@ class FromToLineNoTest(unittest.TestCase):
         #             3,
         #             4)
         discard = stmts[2]
-        self.assertIsInstance(discard, nodes.Discard)
+        self.assertIsInstance(discard, nodes.Expr)
         self.assertEqual(discard.fromlineno, 10)
         self.assertEqual(discard.tolineno, 13)
         callfunc = discard.value
-        self.assertIsInstance(callfunc, nodes.CallFunc)
+        self.assertIsInstance(callfunc, nodes.Call)
         self.assertEqual(callfunc.fromlineno, 10)
         self.assertEqual(callfunc.tolineno, 13)
         name = callfunc.func
@@ -95,7 +97,7 @@ class FromToLineNoTest(unittest.TestCase):
         #                   c):
         #        return a + b + c
         function = stmts[3]
-        self.assertIsInstance(function, nodes.Function)
+        self.assertIsInstance(function, nodes.FunctionDef)
         self.assertEqual(function.fromlineno, 15)
         self.assertEqual(function.tolineno, 18)
         return_ = function.body[0]
@@ -109,14 +111,15 @@ class FromToLineNoTest(unittest.TestCase):
                           '(no line number on function args)')
 
     def test_decorated_function_lineno(self):
-        astroid = test_utils.build_module('''
+        astroid = builder.parse('''
             @decorator
             def function(
                 arg):
                 print (arg)
             ''', __name__)
         function = astroid['function']
-        self.assertEqual(function.fromlineno, 3) # XXX discussable, but that's what is expected by pylint right now
+        # XXX discussable, but that's what is expected by pylint right now
+        self.assertEqual(function.fromlineno, 3)
         self.assertEqual(function.tolineno, 5)
         self.assertEqual(function.decorators.fromlineno, 2)
         self.assertEqual(function.decorators.tolineno, 2)
@@ -134,7 +137,7 @@ class FromToLineNoTest(unittest.TestCase):
         #                 object):
         #       pass
         class_ = stmts[4]
-        self.assertIsInstance(class_, nodes.Class)
+        self.assertIsInstance(class_, nodes.ClassDef)
         self.assertEqual(class_.fromlineno, 20)
         self.assertEqual(class_.tolineno, 22)
         self.assertEqual(class_.blockstart_tolineno, 21)
@@ -172,7 +175,7 @@ class FromToLineNoTest(unittest.TestCase):
             else:
               print ("bouh")
             '''):
-            astroid = test_utils.build_module(code, __name__)
+            astroid = builder.parse(code, __name__)
             stmt = astroid.body[0]
             self.assertEqual(stmt.fromlineno, 2)
             self.assertEqual(stmt.tolineno, 6)
@@ -181,7 +184,7 @@ class FromToLineNoTest(unittest.TestCase):
             self.assertEqual(stmt.orelse[0].tolineno, 6)
 
     def test_try_except_lineno(self):
-        astroid = test_utils.build_module('''
+        astroid = builder.parse('''
             try:
               print (a)
             except:
@@ -202,7 +205,7 @@ class FromToLineNoTest(unittest.TestCase):
 
 
     def test_try_finally_lineno(self):
-        astroid = test_utils.build_module('''
+        astroid = builder.parse('''
             try:
               print (a)
             finally:
@@ -217,7 +220,7 @@ class FromToLineNoTest(unittest.TestCase):
 
 
     def test_try_finally_25_lineno(self):
-        astroid = test_utils.build_module('''
+        astroid = builder.parse('''
             try:
               print (a)
             except:
@@ -234,7 +237,7 @@ class FromToLineNoTest(unittest.TestCase):
 
 
     def test_with_lineno(self):
-        astroid = test_utils.build_module('''
+        astroid = builder.parse('''
             from __future__ import with_statement
             with file("/tmp/pouet") as f:
                 print (f)
@@ -251,35 +254,28 @@ class BuilderTest(unittest.TestCase):
         self.builder = builder.AstroidBuilder()
 
     def test_data_build_null_bytes(self):
-        with self.assertRaises(builder.AstroidBuildingException):
+        with self.assertRaises(exceptions.AstroidSyntaxError):
             self.builder.string_build('\x00')
 
+    def test_data_build_invalid_x_escape(self):
+        with self.assertRaises(exceptions.AstroidSyntaxError):
+            self.builder.string_build('"\\x1"')
+
     def test_missing_newline(self):
         """check that a file with no trailing new line is parseable"""
         resources.build_file('data/noendingnewline.py')
 
     def test_missing_file(self):
-        with self.assertRaises(builder.AstroidBuildingException):
+        with self.assertRaises(exceptions.AstroidBuildingError):
             resources.build_file('data/inexistant.py')
 
     def test_inspect_build0(self):
         """test astroid tree build from a living object"""
         builtin_ast = MANAGER.ast_from_module_name(BUILTINS)
-        if not IS_PY3:
-            fclass = builtin_ast['file']
-            self.assertIn('name', fclass)
-            self.assertIn('mode', fclass)
-            self.assertIn('read', fclass)
-            self.assertTrue(fclass.newstyle)
-            self.assertTrue(fclass.pytype(), '%s.type' % BUILTINS)
-            self.assertIsInstance(fclass['read'], nodes.Function)
-            # check builtin function has args.args == None
-            dclass = builtin_ast['dict']
-            self.assertIsNone(dclass['has_key'].args.args)
         # just check type and object are there
         builtin_ast.getattr('type')
         objectastroid = builtin_ast.getattr('object')[0]
-        self.assertIsInstance(objectastroid.getattr('__new__')[0], nodes.Function)
+        self.assertIsInstance(objectastroid.getattr('__new__')[0], nodes.FunctionDef)
         # check open file alias
         builtin_ast.getattr('open')
         # check 'help' is there (defined dynamically by site.py)
@@ -290,18 +286,17 @@ class BuilderTest(unittest.TestCase):
         self.assertIsInstance(builtin_ast['None'], nodes.Const)
         self.assertIsInstance(builtin_ast['True'], nodes.Const)
         self.assertIsInstance(builtin_ast['False'], nodes.Const)
-        if IS_PY3:
-            self.assertIsInstance(builtin_ast['Exception'], nodes.Class)
-            self.assertIsInstance(builtin_ast['NotImplementedError'], nodes.Class)
-        else:
-            self.assertIsInstance(builtin_ast['Exception'], nodes.From)
-            self.assertIsInstance(builtin_ast['NotImplementedError'], nodes.From)
+        self.assertIsInstance(builtin_ast['Exception'], nodes.ClassDef)
+        self.assertIsInstance(builtin_ast['NotImplementedError'], nodes.ClassDef)
 
     def test_inspect_build1(self):
         time_ast = MANAGER.ast_from_module_name('time')
         self.assertTrue(time_ast)
         self.assertEqual(time_ast['time'].args.defaults, [])
 
+    if os.name == 'java':
+        test_inspect_build1 = unittest.expectedFailure(test_inspect_build1)
+
     def test_inspect_build2(self):
         """test astroid tree build from a living object"""
         try:
@@ -320,8 +315,8 @@ class BuilderTest(unittest.TestCase):
     @test_utils.require_version(maxver='3.0')
     def test_inspect_build_instance(self):
         """test astroid tree build from a living object"""
-        import exceptions
-        builtin_ast = self.builder.inspect_build(exceptions)
+        import exceptions as builtin_exceptions
+        builtin_ast = self.builder.inspect_build(builtin_exceptions)
         fclass = builtin_ast['OSError']
         # things like OSError.strerror are now (2.5) data descriptors on the
         # class instead of entries in the __dict__ of an instance
@@ -333,17 +328,17 @@ class BuilderTest(unittest.TestCase):
     def test_inspect_build_type_object(self):
         builtin_ast = MANAGER.ast_from_module_name(BUILTINS)
 
-        infered = list(builtin_ast.igetattr('object'))
-        self.assertEqual(len(infered), 1)
-        infered = infered[0]
-        self.assertEqual(infered.name, 'object')
-        infered.as_string() # no crash test
+        inferred = list(builtin_ast.igetattr('object'))
+        self.assertEqual(len(inferred), 1)
+        inferred = inferred[0]
+        self.assertEqual(inferred.name, 'object')
+        inferred.as_string() # no crash test
 
-        infered = list(builtin_ast.igetattr('type'))
-        self.assertEqual(len(infered), 1)
-        infered = infered[0]
-        self.assertEqual(infered.name, 'type')
-        infered.as_string() # no crash test
+        inferred = list(builtin_ast.igetattr('type'))
+        self.assertEqual(len(inferred), 1)
+        inferred = inferred[0]
+        self.assertEqual(inferred.name, 'type')
+        inferred.as_string() # no crash test
 
     def test_inspect_transform_module(self):
         # ensure no cached version of the time module
@@ -360,13 +355,16 @@ class BuilderTest(unittest.TestCase):
             MANAGER.unregister_transform(nodes.Module, transform_time)
 
     def test_package_name(self):
-        """test base properties and method of a astroid module"""
+        """test base properties and method of an astroid module"""
         datap = resources.build_file('data/__init__.py', 'data')
         self.assertEqual(datap.name, 'data')
         self.assertEqual(datap.package, 1)
         datap = resources.build_file('data/__init__.py', 'data.__init__')
         self.assertEqual(datap.name, 'data')
         self.assertEqual(datap.package, 1)
+        datap = resources.build_file('data/tmp__init__.py', 'data.tmp__init__')
+        self.assertEqual(datap.name, 'data.tmp__init__')
+        self.assertEqual(datap.package, 0)
 
     def test_yield_parent(self):
         """check if we added discard nodes as yield parent (w/ compiler)"""
@@ -376,12 +374,12 @@ class BuilderTest(unittest.TestCase):
                 if noe:
                     yield more
         """
-        func = test_utils.extract_node(code)
-        self.assertIsInstance(func, nodes.Function)
+        func = builder.extract_node(code)
+        self.assertIsInstance(func, nodes.FunctionDef)
         stmt = func.body[0]
-        self.assertIsInstance(stmt, nodes.Discard)
+        self.assertIsInstance(stmt, nodes.Expr)
         self.assertIsInstance(stmt.value, nodes.Yield)
-        self.assertIsInstance(func.body[1].body[0], nodes.Discard)
+        self.assertIsInstance(func.body[1].body[0], nodes.Expr)
         self.assertIsInstance(func.body[1].body[0].value, nodes.Yield)
 
     def test_object(self):
@@ -410,15 +408,10 @@ class BuilderTest(unittest.TestCase):
             class F:
                 "new style"
         '''
-        mod_ast = test_utils.build_module(data, __name__)
-        if IS_PY3:
-            self.assertTrue(mod_ast['A'].newstyle)
-            self.assertTrue(mod_ast['B'].newstyle)
-            self.assertTrue(mod_ast['E'].newstyle)
-        else:
-            self.assertFalse(mod_ast['A'].newstyle)
-            self.assertFalse(mod_ast['B'].newstyle)
-            self.assertFalse(mod_ast['E'].newstyle)
+        mod_ast = builder.parse(data, __name__)
+        self.assertTrue(mod_ast['A'].newstyle)
+        self.assertTrue(mod_ast['B'].newstyle)
+        self.assertTrue(mod_ast['E'].newstyle)
         self.assertTrue(mod_ast['C'].newstyle)
         self.assertTrue(mod_ast['D'].newstyle)
         self.assertTrue(mod_ast['F'].newstyle)
@@ -435,14 +428,14 @@ class BuilderTest(unittest.TestCase):
                 global CSTE2
                 print (CSTE)
         '''
-        astroid = test_utils.build_module(data, __name__)
+        astroid = builder.parse(data, __name__)
         self.assertEqual(len(astroid.getattr('CSTE')), 2)
-        self.assertIsInstance(astroid.getattr('CSTE')[0], nodes.AssName)
+        self.assertIsInstance(astroid.getattr('CSTE')[0], nodes.AssignName)
         self.assertEqual(astroid.getattr('CSTE')[0].fromlineno, 2)
         self.assertEqual(astroid.getattr('CSTE')[1].fromlineno, 6)
-        with self.assertRaises(NotFoundError):
+        with self.assertRaises(exceptions.AttributeInferenceError):
             astroid.getattr('CSTE2')
-        with self.assertRaises(InferenceError):
+        with self.assertRaises(exceptions.InferenceError):
             next(astroid['global_no_effect'].ilookup('CSTE2'))
 
     def test_socket_build(self):
@@ -452,7 +445,6 @@ class BuilderTest(unittest.TestCase):
         # the socket module) but the last one as those attributes dynamically
         # set and astroid is missing this.
         for fclass in astroid.igetattr('socket'):
-            #print fclass.root().name, fclass.name, fclass.lineno
             self.assertIn('connect', fclass)
             self.assertIn('send', fclass)
             self.assertIn('close', fclass)
@@ -460,48 +452,48 @@ class BuilderTest(unittest.TestCase):
 
     def test_gen_expr_var_scope(self):
         data = 'l = list(n for n in range(10))\n'
-        astroid = test_utils.build_module(data, __name__)
+        astroid = builder.parse(data, __name__)
         # n unavailable outside gen expr scope
         self.assertNotIn('n', astroid)
         # test n is inferable anyway
         n = test_utils.get_name_node(astroid, 'n')
         self.assertIsNot(n.scope(), astroid)
         self.assertEqual([i.__class__ for i in n.infer()],
-                         [YES.__class__])
+                         [util.Uninferable.__class__])
 
     def test_no_future_imports(self):
-        mod = test_utils.build_module("import sys")
+        mod = builder.parse("import sys")
         self.assertEqual(set(), mod.future_imports)
 
     def test_future_imports(self):
-        mod = test_utils.build_module("from __future__ import print_function")
-        self.assertEqual(set(['print_function']), mod.future_imports)
+        mod = builder.parse("from __future__ import print_function")
+        self.assertEqual({'print_function'}, mod.future_imports)
 
     def test_two_future_imports(self):
-        mod = test_utils.build_module("""
+        mod = builder.parse("""
             from __future__ import print_function
             from __future__ import absolute_import
             """)
-        self.assertEqual(set(['print_function', 'absolute_import']), mod.future_imports)
+        self.assertEqual({'print_function', 'absolute_import'}, mod.future_imports)
 
-    def test_infered_build(self):
+    def test_inferred_build(self):
         code = '''
             class A: pass
             A.type = "class"
 
-            def A_ass_type(self):
+            def A_assign_type(self):
                 print (self)
-            A.ass_type = A_ass_type
+            A.assign_type = A_assign_type
             '''
-        astroid = test_utils.build_module(code)
+        astroid = builder.parse(code)
         lclass = list(astroid.igetattr('A'))
         self.assertEqual(len(lclass), 1)
         lclass = lclass[0]
-        self.assertIn('ass_type', lclass.locals)
+        self.assertIn('assign_type', lclass.locals)
         self.assertIn('type', lclass.locals)
 
     def test_augassign_attr(self):
-        test_utils.build_module("""
+        builder.parse("""
             class Counter:
                 v = 0
                 def inc(self):
@@ -510,15 +502,16 @@ class BuilderTest(unittest.TestCase):
         # TODO: Check self.v += 1 generate AugAssign(AssAttr(...)),
         # not AugAssign(GetAttr(AssName...))
 
-    def test_infered_dont_pollute(self):
+    def test_inferred_dont_pollute(self):
         code = '''
             def func(a=None):
                 a.custom_attr = 0
             def func2(a={}):
                 a.custom_attr = 0
             '''
-        test_utils.build_module(code)
+        builder.parse(code)
         nonetype = nodes.const_factory(None)
+        # pylint: disable=no-member; Infers two potential values
         self.assertNotIn('custom_attr', nonetype.locals)
         self.assertNotIn('custom_attr', nonetype.instance_attrs)
         nonetype = nodes.const_factory({})
@@ -527,13 +520,13 @@ class BuilderTest(unittest.TestCase):
 
     def test_asstuple(self):
         code = 'a, b = range(2)'
-        astroid = test_utils.build_module(code)
+        astroid = builder.parse(code)
         self.assertIn('b', astroid.locals)
         code = '''
             def visit_if(self, node):
                 node.test, body = node.tests[0]
             '''
-        astroid = test_utils.build_module(code)
+        astroid = builder.parse(code)
         self.assertIn('body', astroid['visit_if'].locals)
 
     def test_build_constants(self):
@@ -544,7 +537,7 @@ class BuilderTest(unittest.TestCase):
                 return
                 return 'None'
             '''
-        astroid = test_utils.build_module(code)
+        astroid = builder.parse(code)
         none, nothing, chain = [ret.value for ret in astroid.body[0].body]
         self.assertIsInstance(none, nodes.Const)
         self.assertIsNone(none.value)
@@ -552,18 +545,13 @@ class BuilderTest(unittest.TestCase):
         self.assertIsInstance(chain, nodes.Const)
         self.assertEqual(chain.value, 'None')
 
-    def test_lgc_classproperty(self):
-        '''test expected values of constants after rebuilding'''
-        code = '''
-            from logilab.common.decorators import classproperty
-
-            class A(object):
-                @classproperty
-                def hop(cls): #@
-                    return None
-            '''
-        method = test_utils.extract_node(code)
-        self.assertEqual('classmethod', method.type)
+    def test_not_implemented(self):
+        node = builder.extract_node('''
+        NotImplemented #@
+        ''')
+        inferred = next(node.infer())
+        self.assertIsInstance(inferred, nodes.Const)
+        self.assertEqual(inferred.value, NotImplemented)
 
 
 class FileBuildTest(unittest.TestCase):
@@ -571,7 +559,7 @@ class FileBuildTest(unittest.TestCase):
         self.module = resources.build_file('data/module.py', 'data.module')
 
     def test_module_base_props(self):
-        """test base properties and method of a astroid module"""
+        """test base properties and method of an astroid module"""
         module = self.module
         self.assertEqual(module.name, 'data.module')
         self.assertEqual(module.doc, "test module for astroid\n")
@@ -587,19 +575,19 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(module.statement(), module)
 
     def test_module_locals(self):
-        """test the 'locals' dictionary of a astroid module"""
+        """test the 'locals' dictionary of an astroid module"""
         module = self.module
         _locals = module.locals
         self.assertIs(_locals, module.globals)
         keys = sorted(_locals.keys())
-        should = ['MY_DICT', 'YO', 'YOUPI',
-                '__revision__', 'global_access', 'modutils', 'four_args',
-                 'os', 'redirect', 'pb', 'LocalsVisitor', 'ASTWalker']
+        should = ['MY_DICT', 'NameNode', 'YO', 'YOUPI',
+                  '__revision__', 'global_access', 'modutils', 'four_args',
+                  'os', 'redirect']
         should.sort()
-        self.assertEqual(keys, should)
+        self.assertEqual(keys, sorted(should))
 
     def test_function_base_props(self):
-        """test base properties and method of a astroid function"""
+        """test base properties and method of an astroid function"""
         module = self.module
         function = module['global_access']
         self.assertEqual(function.name, 'global_access')
@@ -613,42 +601,46 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(function.type, 'function')
 
     def test_function_locals(self):
-        """test the 'locals' dictionary of a astroid function"""
+        """test the 'locals' dictionary of an astroid function"""
         _locals = self.module['global_access'].locals
         self.assertEqual(len(_locals), 4)
         keys = sorted(_locals.keys())
         self.assertEqual(keys, ['i', 'key', 'local', 'val'])
 
     def test_class_base_props(self):
-        """test base properties and method of a astroid class"""
+        """test base properties and method of an astroid class"""
         module = self.module
         klass = module['YO']
         self.assertEqual(klass.name, 'YO')
-        self.assertEqual(klass.doc, 'hehe')
+        self.assertEqual(klass.doc, 'hehe\n    haha')
         self.assertEqual(klass.fromlineno, 25)
         self.assertTrue(klass.parent)
         self.assertEqual(klass.frame(), klass)
         self.assertEqual(klass.parent.frame(), module)
         self.assertEqual(klass.root(), module)
         self.assertEqual(klass.basenames, [])
-        if IS_PY3:
-            self.assertTrue(klass.newstyle)
-        else:
-            self.assertFalse(klass.newstyle)
+        self.assertTrue(klass.newstyle)
 
     def test_class_locals(self):
-        """test the 'locals' dictionary of a astroid class"""
+        """test the 'locals' dictionary of an astroid class"""
         module = self.module
         klass1 = module['YO']
         locals1 = klass1.locals
         keys = sorted(locals1.keys())
-        self.assertEqual(keys, ['__init__', 'a'])
+        assert_keys = ['__init__', '__module__', '__qualname__', 'a']
+        if sys.version_info < (3, 3):
+            assert_keys.pop(assert_keys.index('__qualname__'))
+        self.assertEqual(keys, assert_keys)
         klass2 = module['YOUPI']
         locals2 = klass2.locals
         keys = locals2.keys()
-        self.assertEqual(sorted(keys),
-                         ['__init__', 'class_attr', 'class_method',
-                         'method', 'static_method'])
+        assert_keys = [
+            '__init__', '__module__', '__qualname__', 'class_attr', 'class_method',
+            'method', 'static_method'
+        ]
+        if sys.version_info < (3, 3):
+            assert_keys.pop(assert_keys.index('__qualname__'))
+        self.assertEqual(sorted(keys), assert_keys)
 
     def test_class_instance_attrs(self):
         module = self.module
@@ -665,14 +657,14 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(klass2.basenames, ['YO'])
 
     def test_method_base_props(self):
-        """test base properties and method of a astroid method"""
+        """test base properties and method of an astroid method"""
         klass2 = self.module['YOUPI']
         # "normal" method
         method = klass2['method']
         self.assertEqual(method.name, 'method')
         self.assertEqual([n.name for n in method.args.args], ['self'])
-        self.assertEqual(method.doc, 'method test')
-        self.assertEqual(method.fromlineno, 47)
+        self.assertEqual(method.doc, 'method\n        test')
+        self.assertEqual(method.fromlineno, 48)
         self.assertEqual(method.type, 'method')
         # class method
         method = klass2['class_method']
@@ -684,7 +676,7 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(method.type, 'staticmethod')
 
     def test_method_locals(self):
-        """test the 'locals' dictionary of a astroid method"""
+        """test the 'locals' dictionary of an astroid method"""
         method = self.module['YOUPI']['method']
         _locals = method.locals
         keys = sorted(_locals)
@@ -692,8 +684,12 @@ class FileBuildTest(unittest.TestCase):
             self.assertEqual(len(_locals), 5)
             self.assertEqual(keys, ['a', 'autre', 'b', 'local', 'self'])
         else:# ListComp variables are no more accessible outside
-            self.assertEqual(len(_locals), 3)
-            self.assertEqual(keys, ['autre', 'local', 'self'])
+            self.assertEqual(len(_locals), 4)
+            self.assertEqual(keys, ['__class__', 'autre', 'local', 'self'])
+
+    def test_unknown_encoding(self):
+        with self.assertRaises(exceptions.AstroidSyntaxError):
+            resources.build_file('data/invalid_encoding.py')
 
 
 class ModuleBuildTest(resources.SysPathSetup, FileBuildTest):
@@ -701,58 +697,19 @@ class ModuleBuildTest(resources.SysPathSetup, FileBuildTest):
     def setUp(self):
         super(ModuleBuildTest, self).setUp()
         abuilder = builder.AstroidBuilder()
-        import data.module
-        self.module = abuilder.module_build(data.module, 'data.module')
+        try:
+            import data.module
+        except ImportError:
+            # Make pylint happy.
+            self.skipTest('Unable to load data.module')
+        else:
+            self.module = abuilder.module_build(data.module, 'data.module')
 
-@unittest.skipIf(IS_PY3, "guess_encoding not used on Python 3")
-class TestGuessEncoding(unittest.TestCase):
-    def setUp(self):
-        self.guess_encoding = builder._guess_encoding
 
-    def testEmacs(self):
-        e = self.guess_encoding('# -*- coding: UTF-8  -*-')
-        self.assertEqual(e, 'UTF-8')
-        e = self.guess_encoding('# -*- coding:UTF-8 -*-')
-        self.assertEqual(e, 'UTF-8')
-        e = self.guess_encoding('''
-        ### -*- coding: ISO-8859-1  -*-
-        ''')
-        self.assertEqual(e, 'ISO-8859-1')
-        e = self.guess_encoding('''
-
-        ### -*- coding: ISO-8859-1  -*-
-        ''')
-        self.assertIsNone(e)
-
-    def testVim(self):
-        e = self.guess_encoding('# vim:fileencoding=UTF-8')
-        self.assertEqual(e, 'UTF-8')
-        e = self.guess_encoding('''
-        ### vim:fileencoding=ISO-8859-1
-        ''')
-        self.assertEqual(e, 'ISO-8859-1')
-        e = self.guess_encoding('''
-
-        ### vim:fileencoding= ISO-8859-1
-        ''')
-        self.assertIsNone(e)
-
-    def test_wrong_coding(self):
-        # setting "coding" varaible
-        e = self.guess_encoding("coding = UTF-8")
-        self.assertIsNone(e)
-        # setting a dictionnary entry
-        e = self.guess_encoding("coding:UTF-8")
-        self.assertIsNone(e)
-        # setting an arguement
-        e = self.guess_encoding("def do_something(a_word_with_coding=None):")
-        self.assertIsNone(e)
-
-    def testUTF8(self):
-        e = self.guess_encoding('\xef\xbb\xbf any UTF-8 data')
-        self.assertEqual(e, 'UTF-8')
-        e = self.guess_encoding(' any UTF-8 data \xef\xbb\xbf')
-        self.assertIsNone(e)
+def test_module_build_dunder_file():
+    """Test that module_build() can work with modules that have the *__file__* attribute"""
+    module = builder.AstroidBuilder().module_build(collections)
+    assert module.path[0] == collections.__file__
 
 
 if __name__ == '__main__':
